;+
;procedure: cotrans
;
;Purpose: geophysical coordinate transformations
;
;         GEI<-->GSE;
;         GSE<-->GSM;
;
;         interpolates the spinphase, right ascension, declination
;
;inputs
;
;	name_in 	... data in the input coordinate system (t-plot variable name)
;   name_out    ... variable name for output            (t-plot variable name)
;
;keywords:
;   TRANSFORMATIONS
;
;
;	/GEI2GSE
;	/GSE2GEI
;
;	/GSE2GSM
;	/GSM2GSE
;
;Examples:
;
;
;      cotrans('tha_fgl_gse','tha_fgl_gsm',/GSE2GSM)
;      cotrans('tha_fgl_gsm','tha_fgl_gse',/GSM2GSE)
;
;      cotrans('tha_fgl_gse','tha_fgl_gei',/GSE2GEI)
;      cotrans('tha_fgl_gei','tha_fgl_gse',/GEI2GSE)
;
;Notes: under construction!!
;
;Written by: Hannes Schwarzl
;
; $LastChangedBy: kenb-mac $
; $LastChangedDate: 2007-02-11 14:04:35 -0800 (Sun, 11 Feb 2007) $
; $LastChangedRevision: 375 $
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/cotrans/cotrans.pro $
;-
pro cotrans, name_in,name_out,GSM2GSE=GSM2GSE,GSE2GEI=GSE2GEI,GSE2GSM=GSE2GSM,GEI2GSE=GEI2GSE


cotrans_lib

;PRINT,'will run faster soon ...'

; get the data using t-plot name
get_data,name_in,data=data_in


is_valid_keyws=0





;GSE GSM
if keyword_set(GSE2GSM) then begin
	is_valid_keyws=1
	sub_GSE2GSM,data_in,data_conv
endif

if keyword_set(GSM2GSE) then begin
	is_valid_keyws=1
	sub_GSE2GSM,data_in,data_conv,/GSM2GSE
endif



;GEI GSE
if keyword_set(GEI2GSE) then begin
	is_valid_keyws=1
	sub_GEI2GSE,data_in,data_conv
endif

if keyword_set(GSE2GEI) then begin
	is_valid_keyws=1
	sub_GEI2GSE,data_in,data_conv,/GSE2GEI
endif



if (is_valid_keyws eq 0) then begin
	PRINT,'Not a valid kombination of input arguments'
endif

store_data,name_out,data=data_conv

;RETURN, data_conv
end












