;+
;FUNCTION file_retrieve(pathnames)
;
;$LastChangedBy: davin-win $
;$LastChangedDate: 2007-02-07 04:33:51 -0800 (Wed, 07 Feb 2007) $
;$LastChangedRevision: 314 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/ssl_general/trunk/misc/file_retrieve.pro $
;-
function file_retrieve,pathnames,  structure_format=structure_format, $
    local_data_dir=local_data_dir,remote_data_dir=remote_data_dir, $
    no_download=no_download,no_update=no_update,verbose=verbose,progress=progress


if keyword_set(structure_format) then begin
   str= {   $
      retrieve_struct,       $
      init:0,                $
      local_data_dir:'',     $
      remote_data_dir:'',    $
      progress: 1    ,      $
      no_download:0   ,      $
      no_update:0     ,      $
      verbose:2              $
   }
   return, str
endif

if not keyword_set(local_data_dir) then   local_data_dir = ''
if not keyword_set(remote_data_dir) then   remote_data_dir = ''
vb = keyword_set(verbose) ? verbose : 0
if n_elements(progress) eq 0 then progress=1

; krb: use concat_dir because local_data_dir may not end with a slash
; del: Changed "concat_dir" to standard IDL routione: "filepath" so that the SSW library is not required for operation.
; BTW: There are still other routines that assume the '/' character is already appended to the *_data_dir
;         It is not unreasonable to assume that users can supply proper input.
fullnames = filepath(root_dir=local_data_dir, pathnames)
n0 = n_elements(fullnames)

if keyword_set(remote_data_dir) and  not keyword_set(no_download) then begin
    if vb ge 2 then  print,ptrace() ,string(format="(i0,' files from ',a)",n0,remote_data_dir)
    for i = 0,n0-1 do begin
        fn = fullnames[i]
        pn = pathnames[i]
        if keyword_set(no_update) and file_test(fn,/regular) then continue
        dirpath = file_dirname(fn)
        if not file_test(dirpath,/directory) then  file_mkdir,dirpath
        if vb ge 2 then print,ptrace()+string(format="(i0,' of ',i0,' :  ',a,' to ',a)",i+1,n0,remote_data_dir+pn,dirpath)
        wait,.01                           ; wait short interval to allow screen updates
        sock_copy,remote_data_dir+pn,out_dir = dirpath,verbose= (vb ge 4),progress=progress,err=err
        if vb ge 1 and keyword_set(err) then print,ptrace(),err
    endfor
endif


; The following bit of code should find the highest version number if globbing is used.

for i=0,n_elements(fullnames)-1 do begin
   ff = file_search(fullnames[i],count=c)
   case c of
   0:    if vb ge 4 then print,ptrace(),'No matching file: ',fullnames[i]
   1:    begin
           fullnames[i] = ff[0]
           if vb ge 5 then print,ptrace(),'Found: ',fullnames[i]
         end
   else: begin
           fullnames[i] = (reverse(ff))[0]   ; Use highest version number?
           if vb ge 2 then print,ptrace(),'Multiple matches found for:',fullnames[i]
         end
   endcase
endfor

return,fullnames
end

