;+
;NAME:
; thm_ui_valid_dtype
;PURPOSE:
; get valid datatype names from thm_load_* routines, using the
; valid_names keyword and returns a list of data types that can be
; loaded for each instrument.
;CALLING SEQUENCE:
; datalist = thm_ui_valid_dtype(instrument)
;INPUT:
; none
;OUTPUT:
; datalist = a list of the datatypes that can be loaded
; instrument = the instrument responsible for this datatype, note that
;    if there is no way to get at the valid_names from the thm_load
;    procedure for a given instrument, then the instrument = the
;    datatype. 
;HISTORY:
; 16-jan-2007, jmm, jimm@ssl.berkeley.edu
; 29-jan-2007, jmm, various changes, now station isn't returned, gmag
; and asi output are hard-wired, other changes will ensue when load
; routines are standardized.
; 4-feb-2007, jmm, Rewritten back to the old hard-wired version....
;$LastChangedBy: kenb-mac $
;$LastChangedDate: 2007-01-26 15:52:34 -0800 (Fri, 26 Jan 2007) $
;$LastChangedRevision: 241 $
;$URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/common/thm_ui_valid_dtype.pro $
;-
Function thm_ui_valid_dtype, ilist
  

  dtyp0 = ['asi', 'ask', $
           'efi', 'fbk', 'fft', 'fgm', $
           'fit', 'gmag', 'mom', 'scm', 'sst', 'state']
  dtyp = 'thm_load_'+dtyp0
  ndtyp = n_elements(dtyp)
  resolve_routine, dtyp, /no_recompile
  For j = 0, ndtyp-1 Do Begin
    call_procedure, dtyp[j], /valid_names, datatype = vvv
    dlist_j = vvv
    ilist_j = replicate(dtyp0[j], n_elements(vvv))
    If(j Eq 0) Then Begin
      ilist = temporary(ilist_j)
      dlist = temporary(dlist_j)
    Endif Else Begin
      ilist = [temporary(ilist), temporary(ilist_j)]
      dlist = [temporary(dlist), temporary(dlist_j)]
    Endelse
  Endfor
  Return, dlist
End
