;+
;	Batch File: THM_CRIB_FBK
;
;	Purpose:  Demonstrate the loading, calibration, and plotting
;		of THEMIS FBK (Filter Bank) spectral data.
;
;	Calling Sequence:
;	.run thm_crib_fbk, or using cut-and-paste.
;
;	Arguements:
;   None.
;
;	Notes:
;	None.
;
; $LastChangedBy: kenb-win2000 $
; $LastChangedDate: 2007-02-12 21:01:54 -0800 (Mon, 12 Feb 2007) $
; $LastChangedRevision: 392 $
; $URL $
;-

; FBK FilterBank data example.

; set a few TPLOT options.
tplot_title = 'THEMIS FBK Spectra Examples'
tplot_options, 'title', tplot_title
tplot_options, 'xmargin', [ 15, 10]
tplot_options, 'ymargin', [ 5, 5]

; set the timespan and load the FBK data (FB1, FB2, FBH).
timespan, '2006-08-04', 1.0, /day
thm_load_fbk, /get_support_data

; set the color table.
loadct2, 39

; set some plotting options for the FBK data using OPTIONS.
options, 'th?_fb1', 'colors', [ 1, 2, 3, 4, 5, 6]
options, 'th?_fb1', 'labels', [ '1', '2', '3', '4', '5', '6']
options, 'th?_fb1', 'labflag', 1

options, 'th?_fb2', 'colors', [ 1, 2, 3, 4, 5, 6]
options, 'th?_fb2', 'labels', [ '1', '2', '3', '4', '5', '6']
options, 'th?_fb2', 'labflag', 1

options, 'th?_fbh', 'colors', [ 1, 2]
options, 'th?_fbh', 'labels', [ 'peak', 'avg']
options, 'th?_fbh', 'labflag', 1
options, 'th?_fbh', 'ylog', 1
options, 'th?_fbh', 'yrange', [ 1.0e-5, 1.0]
options, 'th?_fbh', 'ystyle', 1

; TPLOT the RAW FBK data.
tplot, [ 'tha_fb1', 'tha_fb2', 'tha_fbh']
tlimit, '2006-08-04/19:54:00', '2006-08-04/20:30:00'
stop

; calibrate the FBK data.
thm_cal_fbk, /verbose

; plot the RAW and CALIBRATED FBK data for comparison.
tplot, [ 'tha_fb1_raw', 'tha_fb1', 'tha_fb2_raw', 'tha_fb2', 'tha_fbh_raw', 'tha_fbh']
stop

options, 'th?_fb?_src', 'panel_size', 0.5
options, 'th?_fb?_src', 'psym', 4
options, 'th?_fb?_src', 'yrange', [ -1., 16.]
options, 'th?_fb?_src', 'ystyle', 1



tplot, [ 'tha_fb1_raw', 'tha_fb1', 'tha_fb1_src', 'tha_fb2_raw', 'tha_fb2', 'tha_fb2_src', 'tha_fbh_raw', 'tha_fbh']

end

