;+
;	Batch File: THM_CRIB_FIT
;
;	Purpose:  Demonstrate the loading, calibration, and plotting
;		of THEMIS FIT (On-Board E- and B-Field SpinFit) data.
;
;	Calling Sequence:
;	.run thm_crib_fit, or using cut-and-paste.
;
;	Arguements:
;   None.
;
;	Notes:
;	None.
;
;Written by John Bonnell
; $LastChangedBy: kenb-win2000 $
; $LastChangedDate: 2007-02-12 21:01:54 -0800 (Mon, 12 Feb 2007) $
; $LastChangedRevision: 392 $
; $URL $
;-

; FIT On-Board SpinFit data example.

; set a few TPLOT options.
tplot_title = 'THEMIS FIT On-Board Spin Fit Examples'
tplot_options, 'title', tplot_title
tplot_options, 'xmargin', [ 15, 10]
tplot_options, 'ymargin', [ 5, 5]

; set the timespan and load the FIT data.
timespan, '2006-08-08', 1.0, /day
thm_load_fit, /get_support_data

; set the color table.
loadct2, 39

; the FIT data, as loaded by THM_LOAD_FIT isn't very usable or viewable,
; so run THM_CAL_FIT to break out the E and B fits with useful plotting options.

; calibrate the FIT data.
thm_cal_fit, /verbose

; plot the calibrated FIT data.
tplot, [ 'tha_fit_efit', 'tha_fit_e', 'tha_fit_bfit', 'tha_fit_b']

stop

thm_load_state, probe='a'

dsl2gse, 'tha_fit_b', 'tha_spinras', 'tha_spindec', 'tha_fit_b_gse'

cotrans, 'tha_fit_b_gse', 'tha_fit_b_gsm', /GSE2GSM

tplot, [ 'tha_fit_efit', 'tha_fit_e', 'tha_fit_bfit', 'tha_fit_b', $
	'tha_fit_b_gse', 'tha_fit_b_gsm']

end