;+
;Procedure: THM_LOAD_ASK
;
;Purpose:  Loads THEMIS All Sky Keograms
;
;keywords:
;  site  = Observatory name, example, thm_load_gmag, site = 'fykn', the
;          default is 'all', i.e., load all available stations . This
;          can be an array of strings, e.g., ['fykn', 'gako'] or a
;          single string delimited by spaces, e.g., 'fykn gako'
;  datatype = request 'ast' or 'asf', default is 'asf', can also be 'all'.
;  TRANGE= (Optional) Time range of interest  (2 element array), if
;          this is not set, the default is to prompt the user. Note
;          that if the input time range is not a full day, a full
;          day's data is loaded
;  level = the level of the data, the default is 'l2', or level-2
;          data. A string (e.g., 'l2') or an integer can be used. 'all'
;          can be passed in also, to get all levels.
;  /VERBOSE : set to output some useful info
;  /downloadonly, if set, then only download the data, do not load it
;                 into variables.
;  /valid_names, if set, then this will return the valid site, datatype
;                and/or level options in named variables, for example,
;
;                thm_load_gmag, site = xxx, /valid_names
;
;                will return the array of valid sites in the
;                variable xxx
;Example:
;   thg_load_ask
;Notes:
;  This routine is (should be) platform independent.
;
;
; $LastChangedBy: kenb-mac $
; $LastChangedDate: 2007-02-09 14:40:39 -0800 (Fri, 09 Feb 2007) $
; $LastChangedRevision: Added valid_names output option$
; $URL: svn+ssh://thmsvn@ambrosia.ssl.berkeley.edu/repos/thmsoc/trunk/idl/themis/ground/thm_load_ask.pro $
;-

; find the correct file names, based on trange, datatype, and site
function thm_load_ask_relpath, trange=trange, _extra=_extra

   relpath = 'thg/l1/asi/ask/'
   prefix = 'thg_l1_ask_'
   ending = '_v01.cdf'
   return, file_dailynames(relpath,prefix,ending,/YEARDIR,trange=trange)

end

pro thm_load_ask,site = site, datatype = datatype, trange = trange, $
                 level = level, verbose = verbose, $
                 downloadonly = downloadonly, $
                 valid_names = valid_names, $
                 _extra = _extra


  if keyword_set(site) && ~strcmp(strlowcase(site), 'all') then $
     message,/info,'Sorry, Must get all stations for now.'

  thm_load_xxx,sname=site, datatype=datatype, trange=trange, $
               level=level, verbose=verbose, downloadonly=downloadonly, $
               cdf_data=cdf_data,get_cdf_data=arg_present(cdf_data), $
               get_support_data=get_support_data, $
               varnames=varnames, valid_names = valid_names, files=files, $
               vsnames = 'all', $
               type_sname = 'site', $
               vdatatypes = 'ask', $
               vlevels = 'l1', $
               deflevel = 'l1', $
               version = 'v01', $
               relpath_funct = 'thm_load_ask_relpath', $
               _extra = _extra

end

