; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;
; create a RFF file containing a sine test signal
;
; P. Robert, CETP, December 2006
;
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

  PRO crea_sin_test

  PRINT
  PRINT, '------------------------------------------------------------'
  PRINT, 'crea_sin_test: create a RFF file containing sine signal test'
  PRINT, '------------------------------------------------------------'
  PRINT

; compute rff file name

  rffname='sintest.rff'

; Metadata definitions

  cretime= SYSTIME(1)
  datimstr= time_to_str(cretime, FMT=0)
  isodatim= str_to_iso_datim(datimstr)
  satname='Sin test'
  satnum =1
  subtitle=satname
  mode='444'
  forout='(a29,1x,f5.0,1x,f10.6,3(1x,e14.6))'  ; a20 +nbdec +1 for 'Z'

; other parameters

  str_date='2007-02-15'
  julsec1=datesec(str_date)
  Fe= 8.
  dt=1./Fe
  f=0.5
  sphase1=350.D0
  count=2000

; stat file values

  time_state=DBLARR(3)
  spinper =  DBLARR(3)
  spinrasc=  FLTARR(3)
  spindecl=  FLTARR(3)

  FOR n=0,2 DO BEGIN
  time_state(n)= n*dt*20.D0 +julsec1
  spinper(n)=3.01D0 +n*0.000001D0
  spinrasc(n)=67.25 +n*0.01
  spindecl(n)=85.00 +n*0.01
  ENDFOR


; ----------------------------------------------------------------------
; open rff file, and fill file with header

  GET_LUN, unit
  PRINT, FORMAT='(3a,i4,a)', $
         'Creating ',rffname,' on unit',unit,', Please wait...'
  OPENW, unit, rffname

  PRINT, '     Fill file with RFF header...'

  thm_scmwf_wrff_header, unit, rffname, satname, satnum, $
                              mode, forout, subtitle, count

  PRINT, '     Fill file with constant data...'

  thm_scmwf_wrff_condat, unit, time_state, spinper, spinrasc, spindecl

  PRINT, '     Fill file with Time Vector data ...'

  FOR n=0L, count-1 DO BEGIN

; ( 15 digits, 16 max, so micro-sec, one more digit max.)

  sec_day= DOUBLE(n*dt)
  str_time= secofday_to_strtime(sec_day,8)
  datimiso= str_date +'T' +str_time +'Z'

  x= 10.*sin(2.*!pi*f*FLOAT(n)*dt )
  y= 10.*cos(2.*!pi*f*FLOAT(n)*dt )
  z= 40.*cos(2.*!pi*f*FLOAT(n)*dt )

  sphase=sphase1 +360.D*DOUBLE(n)*dt/spinper(0)
  nbtour=FIX(sphase/360.D)
  sphase=sphase-FIX(sphase/360.D)*360.D

  PRINTF, unit, FORMAT=forout, datimiso,Fe,sphase, x, y, z
  ENDFOR


; write RFF EOF on file

  PRINT, '     Write RFF EOF on file ...'
  thm_scmwf_wrff_eof, unit

  FREE_LUN,unit
  PRINT, 'It''s done, RFF file is ready.'

  END

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
