; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;
; take scm data, state_file data and scm_feader data to create
; a RFF file including all we need to calibrate SCM data.
;
; P. Robert, CETP, January 2007
;
; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX

  PRO scmwf_create_rff, rffname,time_scm,val_scm,satname,mode, $
                        time_state, spinpha,spinper,spinrasc,spindecl, $
                        time_head, TMrate
  PRINT
  PRINT, '------------------------------------------------------------'
  PRINT, 'scmwf_create_rff: create a RFF file containing scm waveform '
  PRINT, '------------------------------------------------------------'
  PRINT

; Metadata definitions

  IF (satname EQ 'a') THEN satnum=1
  IF (satname EQ 'b') THEN satnum=2
  IF (satname EQ 'c') THEN satnum=3
  IF (satname EQ 'd') THEN satnum=4
  IF (satname EQ 'e') THEN satnum=5

  subtitle='THEMIS/SCM-wf/'+'th' +satname $
                           +' (#' +STRING(satnum,format='(I1)') +')'
  forout='(a29,1x,f5.0,1x,f10.6,3(1x,e14.6))'  ; a20 +nbdec +1 for 'Z'
  count=N_ELEMENTS(time_scm)

; ISO date time

; Themis time is now from 1970/01/01
; julsec=time_scm  +978307200.D
; julsta=time_head +978307200.D
  isodatim=Julsec_to_isodatim(time_scm,8)

; FOR i=0L,count-1L DO PRINT, time_scm(i),time_scm(i)+978307200.D, $
;              isodatim(i), $ FORMAT='(f15.10,1x,f29.13,1x,a)'

; interpollation of sample rate and spin period, aligned on scm time

  PRINT, 'Sample rate interpolation...'
  scm_rate_interpol, time_head,TMrate,time_scm,SA_rate

  PRINT, 'Spin  phase interpolation...'
  scm_spin_interpol, time_state,spinpha,spinper,time_scm,SP_phase

; ----------------------------------------------------------------------

; open rff file & fill file with header, constant data and indexed data

  GET_LUN, unit

  PRINT, ' '
  PRINT, FORMAT='(3a,i4,a)', $
         'Creating ',rffname,' on unit',unit,', Please wait...'
  OPENW, unit, rffname

  PRINT, '     Fill file with RFF header...'

  thm_scmwf_wrff_header, unit, rffname, satname, satnum, $
                              mode, forout, subtitle, count

  PRINT, '     Fill file with constant data...'

  thm_scmwf_wrff_condat, unit, time_head, spinper, spinrasc, spindecl

  PRINT, '     Fill file with Time Vector data ...'

  FOR n=0L, count-1L DO BEGIN
  PRINTF, unit, FORMAT=forout, isodatim(n),SA_rate(n),SP_phase(n), $
                               val_scm(n,0),val_scm(n,1),val_scm(n,2)
  ENDFOR


; write RFF EOF on file

  PRINT, '     Write RFF EOF on file ...'
  thm_scmwf_wrff_eof, unit

  FREE_LUN, unit

; ----------------------------------------------------------------------

  PRINT, 'It''s done, RFF file created. Nb of vectors written=', count $
       , FORMAT='(a,i9)'
  PRINT, '------------------------------------------------------------'
  PRINT

  END

; XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
